from typing import List

from cadenzaanalytics.data.attribute_group import AttributeGroup
from cadenzaanalytics.data.data_object import DataObject
from cadenzaanalytics.data.extension_type import ExtensionType
from cadenzaanalytics.data.parameter import Parameter


class AnalyticsExtension(DataObject):
    _attribute_mapping = {
        "printName": "_print_name",
        "extensionType": "_extension_type",
        "attributeGroups": "_attribute_groups",
        "parameters": "_parameters"
    }

    def __init__(self,
                 print_name: str,
                 extension_type: ExtensionType,
                 attribute_groups: List[AttributeGroup],
                 parameters: List[Parameter] = None):
        self._print_name = print_name
        self._extension_type = extension_type
        self._attribute_groups = attribute_groups
        self._parameters = parameters

    @property
    def print_name(self) -> str:
        return self._print_name

    @property
    def extension_type(self) -> ExtensionType:
        return self._extension_type



from typing import List

from cadenzaanalytics.data.data_object import DataObject
from cadenzaanalytics.data.data_type import DataType


class AttributeGroup(DataObject):
    _attribute_mapping = {
        "name": "_name",
        "printName": "_print_name",
        "dataTypes": "_data_types",
        "geometryTypes": "_geometry_types",
        "minAttributes": "_min_attributes",
        "maxAttributes": "_max_attributes"
    }

    def __init__(self,
                 name: str,
                 print_name: str,
                 data_types: List[DataType],
                 geometry_types: List = None,
                 min_attributes: int = 0,
                 max_attributes: int = None):
        self._name = name
        self._print_name = print_name
        self._data_types = data_types
        self._geometry_types = geometry_types
        self._min_attributes = min_attributes
        self._max_attributes = max_attributes



import json

from flask import Response
from requests_toolbelt import MultipartEncoder

from cadenzaanalytics.data.data_container_metadata import DataContainerMetadata
from cadenzaanalytics.response.extension_response import ExtensionResponse


# pylint: disable=protected-access
class ExtensionDataResponse(ExtensionResponse):
    def __init__(self, content_type, data_container_name='response-data'):
        self._content_type = content_type
        self._data_container_name = data_container_name

    def _get_response_metadata(self, column_metadata):
        metadata = DataContainerMetadata(self._content_type, self._data_container_name, column_metadata)

        # TODO: refactor: utilize message metadata class
        metadata = {
            "dataContainers": [
                metadata._to_dict()
            ]
        }

        return json.dumps(metadata, default=str)

    def _create_response(self, data, column_metadata=None):
        multipart_response = MultipartEncoder(
            {'metadata': (None, self._get_response_metadata(column_metadata), 'application/json'),
             self._data_container_name: (None, data, self._content_type)}
        )

        return Response(multipart_response.to_string(), mimetype=multipart_response.content_type)



"""The `cadenzaanalytics` module provides functionality to extend the business
and location intelligence software disy Cadenza with analytics extensions, which 
may be used to create Python code, e.g. to create, enrich or visualize data using Python.

.. include:: ../../docs/intro.md
"""
from cadenzaanalytics.cadenza_analytics_extension import CadenzaAnalyticsExtension
from cadenzaanalytics.cadenza_analytics_extension_service import CadenzaAnalyticsExtensionService
from cadenzaanalytics.data.analytics_extension import AnalyticsExtension
from cadenzaanalytics.data.attribute_group import AttributeGroup
from cadenzaanalytics.data.attribute_role import AttributeRole
from cadenzaanalytics.data.column_metadata import ColumnMetadata
from cadenzaanalytics.data.data_type import DataType
from cadenzaanalytics.data.extension_type import ExtensionType
from cadenzaanalytics.data.geometry_type import GeometryType
from cadenzaanalytics.data.measure_aggregation import MeasureAggregation
from cadenzaanalytics.data.parameter import Parameter
from cadenzaanalytics.data.parameter_type import ParameterType
from cadenzaanalytics.request.request_metadata import RequestMetadata
from cadenzaanalytics.response.csv_response import CsvResponse
from cadenzaanalytics.response.error_response import ErrorResponse
from cadenzaanalytics.response.image_response import ImageResponse
